﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.IO;
using System.Reflection;
using System.ServiceProcess;
using System.Configuration.Install;

using Microsoft.Win32;

namespace WinServiceController
{
    public class ServiceControllerUtil:ServiceController 
    {
        /// <summary>
        /// 根据指定服务名创建一个ServiceControllerUtil
        /// </summary>
        /// <param name="service_Name">Windows服务名</param>
        public ServiceControllerUtil(string service_Name):base(service_Name)
        {
        }

        /// <summary>
        /// 获取服务执行程序的路径
        /// </summary>
        public string FilePath
        {
            get
            {
                RegistryKey _Key = Registry.LocalMachine.OpenSubKey(@"SYSTEM\ControlSet001\Services\" + base.ServiceName);
                if (_Key != null)
                {
                    object _ObjPath = _Key.GetValue("ImagePath");
                    if (_ObjPath != null) return _ObjPath.ToString();
                }
                return "";
            }
        }

        /// <summary>
        /// 获取服务的描述内容
        /// </summary>
        public string Description
        {
            get
            {
                RegistryKey _Key = Registry.LocalMachine.OpenSubKey(@"SYSTEM\ControlSet001\Services\" + base.ServiceName);
                if (_Key != null)
                {
                    object _ObjPath = _Key.GetValue("Description");
                    if (_ObjPath != null) return _ObjPath.ToString();
                }
                return "";
            }
        }

        /// <summary>
        /// 卸载服务
        /// </summary>
        /// <param name="p_Path">服务执行文件的路径</param>          
        /// <returns>卸载成功返回""，卸载失败，则返回失败原因</returns>
        public static string UnInstallService(string p_Path)
        {
            if (!File.Exists(p_Path)) return "文件不存在！";
            FileInfo _InstallFile = new FileInfo(p_Path);
            IDictionary _SavedState = new Hashtable();
            try
            {
                AssemblyInstaller _AssemblyInstaller = new AssemblyInstaller(p_Path, new string[] { "/LogFile=" + _InstallFile.DirectoryName + "\\" + _InstallFile.Name.Substring(0, _InstallFile.Name.Length - _InstallFile.Extension.Length) + ".log" });
                _AssemblyInstaller.UseNewContext = true;
                _AssemblyInstaller.Uninstall(_SavedState);
                _AssemblyInstaller.Commit(_SavedState);
                return "";
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }

        /// <summary>
        /// 安装服务
        /// </summary>
        /// <param name="p_Path">服务执行文件的路径</param>
        /// <param name="p_ServiceName">返回安装完成后的服务名</param>
        /// <returns>安装成功返回""，安装服务失败则返回错误信息</returns>
        public static string IntallService(string p_Path, ref string p_ServiceName)
        {
            if (!File.Exists(p_Path)) return "文件不存在！";
            p_ServiceName = "";
            FileInfo _InstallFile = new FileInfo(p_Path);
            IDictionary _SavedState = new Hashtable();
            try
            {
                AssemblyInstaller _AssemblyInstaller = new AssemblyInstaller(p_Path, new string[] { "/LogFile=" + _InstallFile.DirectoryName + "\\" + _InstallFile.Name.Substring(0, _InstallFile.Name.Length - _InstallFile.Extension.Length) + ".log" });
                _AssemblyInstaller.UseNewContext = true;
                _AssemblyInstaller.Install(_SavedState);
                _AssemblyInstaller.Commit(_SavedState);

                Type[] _TypeList = _AssemblyInstaller.Assembly.GetTypes();

                for (int i = 0; i != _TypeList.Length; i++)
                {
                    if (_TypeList[i].BaseType.FullName == "System.Configuration.Install.Installer")
                    {
                        object _InstallObject = System.Activator.CreateInstance(_TypeList[i]);
                        FieldInfo[] _FieldList = _TypeList[i].GetFields(BindingFlags.NonPublic | BindingFlags.Instance);
                        for (int z = 0; z != _FieldList.Length; z++)
                        {
                            if (_FieldList[z].FieldType.FullName == "System.ServiceProcess.ServiceInstaller")
                            {
                                object _ServiceInstall = _FieldList[z].GetValue(_InstallObject);
                                if (_ServiceInstall != null)
                                {
                                    p_ServiceName = ((ServiceInstaller)_ServiceInstall).ServiceName;
                                    return "";
                                }
                            }
                        }
                    }
                }
                return "";
            }
            catch (Exception ex)
            {
                return ex.Message;
            }
        }

        /// <summary>
        /// 获取服务列表（设备驱动服务除外）
        /// </summary>
        /// <returns></returns>
        public static IList<ServiceControllerUtil> GetServicesInfo()
        {
            IList<ServiceControllerUtil> _List = new List<ServiceControllerUtil>();
            ServiceController[] _ServiceList = ServiceController.GetServices();
            for (int i = 0; i != _ServiceList.Length; i++)
            {
                _List.Add(new ServiceControllerUtil(_ServiceList[i].ServiceName));
            }
            return _List;
        }

        /// <summary>
        /// 获取设备驱动服务
        /// </summary>
        /// <returns></returns>
        public static IList<ServiceControllerUtil> GetDevicesInfo()
        {
            IList<ServiceControllerUtil> _List = new List<ServiceControllerUtil>();
            ServiceController[] _ServiceList = ServiceController.GetDevices();
            for (int i = 0; i != _ServiceList.Length; i++)
            {
                _List.Add(new ServiceControllerUtil(_ServiceList[i].ServiceName));
            }
            return _List;
        }
    }
}
